package speiger.src.coreengine.math.vector.bytes;

import java.nio.ByteBuffer;

import speiger.src.coreengine.math.MathUtils;
import speiger.src.coreengine.math.vector.doubles.Vec3d;
import speiger.src.coreengine.math.vector.floats.Vec3f;
import speiger.src.coreengine.math.vector.ints.Vec3i;
import speiger.src.coreengine.math.vector.longs.Vec3l;
import speiger.src.coreengine.math.vector.shorts.Vec3s;

public interface Vec3b extends Vecb
{
	public static final Vec3b ZERO = newVec();
	public static final Vec3b MINUS_ONE = newVec((byte)-1);
	public static final Vec3b ONE = newVec((byte)1);
	
	public static Vec3b newMutable(){return new Vec3bMutable();}
	public static Vec3b newMutable(byte value){return new Vec3bMutable(value);}
	public static Vec3b newMutable(byte x, byte y, byte z){return new Vec3bMutable(x, y, z);}
	public static Vec3b newMutable(Vec3b vec){return newMutable(vec.getX(), vec.getY(), vec.getZ());}
	
	public static Vec3b newVec(){return new Vec3bImmutable();}
	public static Vec3b newVec(byte value){return new Vec3bImmutable(value);}
	public static Vec3b newVec(byte x, byte y, byte z){return new Vec3bImmutable(x, y, z);}
	public static Vec3b newVec(Vec3b vec){return newVec(vec.getX(), vec.getY(), vec.getZ());}
	
	public byte getX();
	public byte getY();
	public byte getZ();
	public Vec3b setX(byte x);
	public Vec3b setY(byte y);
	public Vec3b setZ(byte z);
	@Override
	public default byte[] asArray(){return new byte[]{getX(), getY(), getZ()};}
	
	@Override
	public Vec3b copy();
	@Override
	public default Vec3b abs(){return set((byte)Math.abs(getX()), (byte)Math.abs(getY()), (byte)Math.abs(getZ()));}
	@Override
	public default Vec3b negate(){return set((byte)0, (byte)0, (byte)0);}
	@Override
	public default Vec3b invert(){return set((byte)-getX(), (byte)-getY(), (byte)-getZ());}
	@Override
	public default Vec3b add(byte value){return add(value, value, value);}	
	public default Vec3b add(Vec3b value){return add(value.getX(), value.getY(), value.getZ());}
	public default Vec3b add(byte x, byte y, byte z){return set((byte)(getX() + x), (byte)(getY() + y), (byte)(getZ() + z));}

	@Override
	public default Vec3b sub(byte value){return sub(value, value, value);}
	public default Vec3b sub(Vec3b value){return sub(value.getX(), value.getY(), value.getZ());}
	public default Vec3b sub(byte x, byte y, byte z){return set((byte)(getX() - x), (byte)(getY() - y), (byte)(getZ() - z));}
	
	@Override
	public default Vec3b multiply(byte value){return multiply(value, value, value);}
	public default Vec3b multiply(Vec3b value){return multiply(value.getX(), value.getY(), value.getZ());}
	public default Vec3b multiply(byte x, byte y, byte z){return set((byte)(getX() * x), (byte)(getY() * y), (byte)(getZ() * z));}

	@Override
	public default Vec3b devide(byte value){return devide(value, value, value);}
	public default Vec3b devide(Vec3b value){return devide(value.getX(), value.getY(), value.getZ());}
	public default Vec3b devide(byte x, byte y, byte z){return set((byte)(getX() / x), (byte)(getY() / y), (byte)(getZ() / z));}
	
	@Override
	public default Vec3b set(byte value){return set(value, value, value);}
	public default Vec3b set(Vec3b value){return set(value.getX(), value.getY(), value.getZ());}
	public Vec3b set(byte x, byte y, byte z);
	
	public default double distanceTo(Vec3b value){return distanceTo(value.getX(), value.getY(), value.getZ());}
	public default double distanceTo(byte x, byte y, byte z){return Math.sqrt(distanceToSquared(x, y, z));}
	
	public default long distanceToSquared(Vec3b value){return distanceToSquared(value.getX(), value.getY(), value.getZ());}
	public default long distanceToSquared(byte x, byte y, byte z)
	{
		long xPos = getX() - x;
		long yPos = getY() - y;
		long zPos = getZ() - z;
		return (xPos * xPos) + (yPos * yPos) + (zPos * zPos);
	}
	@Override
	public default long lengthSquared() {return (getX() * getX()) + (getY() * getY()) + (getZ() * getZ());}
	
	public default long dotProduct(Vec3b value){return dotProduct(value.getX(), value.getY(), value.getZ());}
	public default long dotProduct(byte x, byte y, byte z){return (getX() * x) + (getY() * y) + (getZ() * z);}
	
	public default Vec3b min(Vec3b other) {return min(other, this);}
	public default Vec3b min(Vec3b other, Vec3b result){return min(other.getX(), other.getY(), other.getZ(), result);}
	public default Vec3b min(byte x, byte y, byte z) {return min(x, y, z, this);}
	public default Vec3b min(byte x, byte y, byte z, Vec3b result){return result.set((byte)Math.min(getX(), x), (byte)Math.min(getY(), y), (byte)Math.min(getZ(), z));}
	
	public default Vec3b max(Vec3b other) {return max(other, this);}
	public default Vec3b max(Vec3b other, Vec3b result){return max(other.getX(), other.getY(), other.getZ(), result);}
	public default Vec3b max(byte x, byte y, byte z) {return max(x, y, z, this);}
	public default Vec3b max(byte x, byte y, byte z, Vec3b result){return result.set((byte)Math.max(getX(), x), (byte)Math.max(getY(), y), (byte)Math.max(getZ(), z));}
	
	public default Vec3b difference(Vec3b other) {return difference(other, this);}
	public default Vec3b difference(Vec3b other, Vec3b result){return difference(other.getX(), other.getY(), other.getZ(), result);}
	public default Vec3b difference(byte x, byte y, byte z) {return difference(x, y, z, this);}
	public default Vec3b difference(byte x, byte y, byte z, Vec3b result){return result.set((byte)(getX() - x), (byte)(getY() - y), (byte)(getZ() - z));}
	
	@Override
	public default Vec3b clamp(byte min, byte max){return clamp(min, max, ALL);}
	public default Vec3b clamp(byte min, byte max, Vec3b result){return clamp(min, max, result, ALL);}
	@Override
	public default Vec3b clamp(byte min, byte max, int filter){return clamp(min, max, this, filter);}
	public default Vec3b clamp(byte min, byte max, Vec3b result, int filter){ return result.set((filter & X) == 0 ? getX() : MathUtils.clamp(min, max, getX()), (filter & Y) == 0 ? getY() : MathUtils.clamp(min, max, getY()), (filter & Z) == 0 ? getZ() : MathUtils.clamp(min, max, getZ()));}
	
	@Override
	public default Vec3b store(ByteBuffer buffer)
	{
		buffer.put(getX()).put(getY()).put(getZ());
		return this;
	}
	
	@Override
	public default Vec3b load(ByteBuffer buffer)
	{
		return set(buffer.get(), buffer.get(), buffer.get());
	}
	
	@Override
	public default Vec3s asShort(){return isMutable() ? Vec3s.newMutable(getX(), getY(), getZ()) : Vec3s.newVec(getX(), getY(), getZ());}
	@Override
	public default Vec3i asInt(){return isMutable() ? Vec3i.newMutable(getX(), getY(), getZ()) : Vec3i.newVec(getX(), getY(), getZ());}
	@Override
	public default Vec3l asLong(){return isMutable() ? Vec3l.newMutable(getX(), getY(), getZ()) : Vec3l.newVec(getX(), getY(), getZ());}
	@Override
	public default Vec3f asFloat() {return isMutable() ? Vec3f.newMutable(getX(), getY(), getZ()) : Vec3f.newVec(getX(), getY(), getZ());}
	@Override
	public default Vec3d asDouble(){return isMutable() ? Vec3d.newMutable(getX(), getY(), getZ()) : Vec3d.newVec(getX(), getY(), getZ());}
	
	@Override
	public default Vec3b asMutable(){return isMutable() ? this : newMutable(this);}
	@Override
	public default Vec3b asImmutable(){return isMutable() ? newVec(this) : this;}
	@Override
	public default Vec3b copyAsMutable(){return newMutable(this);}
	@Override
	public default Vec3b copyAsImmutable(){return newVec(this);}
}
