package speiger.src.coreengine.utils.io.dataTag.special;

import java.util.UUID;

import speiger.src.coreengine.utils.io.dataTag.DataTag;
import speiger.src.coreengine.utils.io.dataTag.array.ByteArrayTag;
import speiger.src.coreengine.utils.io.dataTag.array.DoubleArrayTag;
import speiger.src.coreengine.utils.io.dataTag.array.FloatArrayTag;
import speiger.src.coreengine.utils.io.dataTag.array.IntArrayTag;
import speiger.src.coreengine.utils.io.dataTag.array.LongArrayTag;
import speiger.src.coreengine.utils.io.dataTag.array.MediumArrayTag;
import speiger.src.coreengine.utils.io.dataTag.array.ShortArrayTag;
import speiger.src.coreengine.utils.io.dataTag.array.StringArrayTag;
import speiger.src.coreengine.utils.io.dataTag.simple.ByteTag;
import speiger.src.coreengine.utils.io.dataTag.simple.DoubleTag;
import speiger.src.coreengine.utils.io.dataTag.simple.FloatTag;
import speiger.src.coreengine.utils.io.dataTag.simple.IntTag;
import speiger.src.coreengine.utils.io.dataTag.simple.LongTag;
import speiger.src.coreengine.utils.io.dataTag.simple.MediumTag;
import speiger.src.coreengine.utils.io.dataTag.simple.PrimitiveTag;
import speiger.src.coreengine.utils.io.dataTag.simple.ShortTag;
import speiger.src.coreengine.utils.io.dataTag.simple.StringTag;

public interface IMapTag extends DataTag
{
	public void setTag(String id, DataTag tag);
	public default void setList(String id, ListTag tag) { if(!tag.isEmpty()) setTag(id, tag); }
	public default void setMap(String id, MapTag tag) { if(!tag.isEmpty()) setTag(id, tag); }
	
	public default void setBoolean(String id, boolean value) {setByte(id, (byte)(value ? 1 : 0), (byte)0);}
	public default void setByte(String id, byte value) {setByte(id, value, (byte)0);}
	public default void setByte(String id, byte value, byte defaultValue)
	{
		if(value != defaultValue)
		{
			setTag(id, new ByteTag(value));
		}
	}
	
	public default void setShort(String id, short value) {setShort(id, value, (short)0);}
	public default void setShort(String id, short value, short defaultValue)
	{
		if(value != defaultValue)
		{
			setTag(id, new ShortTag(value));
		}
	}
	
	public default void setMedium(String id, int value) {setMedium(id, value, 0);}
	public default void setMedium(String id, int value, int defaultValue)
	{
		if(value != defaultValue)
		{
			setTag(id, new MediumTag(value));
		}
	}
	
	public default void setInt(String id, int value) {setInt(id, value, 0);}
	public default void setInt(String id, int value, int defaultValue)
	{
		if(value != defaultValue)
		{
			setTag(id, new IntTag(value));
		}
	}
	
	public default void setLong(String id, long value) {setLong(id, value, 0L);}
	public default void setLong(String id, long value, long defaultValue)
	{
		if(value != defaultValue)
		{
			setTag(id, new LongTag(value));
		}
	}
	
	public default void setFloat(String id, float value) {setFloat(id, value, 0F);}
	public default void setFloat(String id, float value, float defaultValue)
	{
		if(value != defaultValue)
		{
			setTag(id, new FloatTag(value));
		}
	}
	
	public default void setDouble(String id, double value) {setDouble(id, value, 0D);}
	public default void setDouble(String id, double value, double defaultValue)
	{
		if(value != defaultValue)
		{
			setTag(id, new DoubleTag(value));
		}
	}
	
	public default void setString(String id, String value) {setString(id, value, "");}
	public default void setString(String id, String value, String defaultValue)
	{
		if(!value.equals(defaultValue))
		{
			setTag(id, new StringTag(value));
		}
	}
	
	public default void setByteArray(String id, byte[] data) {if(data.length > 0){setTag(id, new ByteArrayTag(data));}}
	public default void setShortArray(String id, short[] data) {if(data.length > 0){setTag(id, new ShortArrayTag(data));}}
	public default void setMediumArray(String id, int[] data) {if(data.length > 0){setTag(id, new MediumArrayTag(data));}}
	public default void setIntArray(String id, int[] data) {if(data.length > 0){setTag(id, new IntArrayTag(data));}}
	public default void setLongArray(String id, long[] data) {if(data.length > 0){setTag(id, new LongArrayTag(data));}}
	public default void setFloatArray(String id, float[] data) {if(data.length > 0){setTag(id, new FloatArrayTag(data));}}
	public default void setDoubleArray(String id, double[] data) {if(data.length > 0){setTag(id, new DoubleArrayTag(data));}}
	public default void setStringArray(String id, String[] data) {if(data.length > 0){setTag(id, new StringArrayTag(data));}}
	
	public default void setUUID(String id, UUID data) { if(data != null) setLongArray(id, new long[]{data.getMostSignificantBits(), data.getLeastSignificantBits()});}
	
	public DataTag get(String id);
	
	public default boolean getBoolean(String id) { return getByte(id, (byte)0) != 0;}
	public default byte getByte(String id) {return getByte(id, (byte)0);}
	public default byte getByte(String id, byte defaultValue)
	{
		DataTag tag = get(id);
		return isPrimitve(tag) ? ((PrimitiveTag)tag).asByte() : defaultValue;
	}
	
	public default short getShort(String id) {return getShort(id, (short)0);}
	public default short getShort(String id, short defaultValue)
	{
		DataTag tag = get(id);
		return isPrimitve(tag) ? ((PrimitiveTag)tag).asShort() : defaultValue;
	}
	
	public default int getMedium(String id) {return getMedium(id, 0);}
	public default int getMedium(String id, int defaultValue)
	{
		DataTag tag = get(id);
		return isPrimitve(tag) ? ((PrimitiveTag)tag).asInt() : defaultValue;
	}
	
	public default int getInt(String id) {return getInt(id, 0);}
	public default int getInt(String id, int defaultValue)
	{
		DataTag tag = get(id);
		return isPrimitve(tag) ? ((PrimitiveTag)tag).asInt() : defaultValue;
	}
	
	public default long getLong(String id) {return getLong(id, 0L);}
	public default long getLong(String id, long defaultValue)
	{
		DataTag tag = get(id);
		return isPrimitve(tag) ? ((PrimitiveTag)tag).asLong() : defaultValue;
	}
	
	public default float getFloat(String id) {return getFloat(id, 0F);}
	public default float getFloat(String id, float defaultValue)
	{
		DataTag tag = get(id);
		return isPrimitve(tag) ? ((PrimitiveTag)tag).asFloat() : defaultValue;
	}
	
	public default double getDouble(String id) {return getDouble(id, 0D);}
	public default double getDouble(String id, double defaultValue)
	{
		DataTag tag = get(id);
		return isPrimitve(tag) ? ((PrimitiveTag)tag).asByte() : defaultValue;
	}
	
	public default String getString(String id) {return getString(id, "");}
	public default String getString(String id, String defaultValue)
	{
		DataTag tag = get(id);
		return isType(tag, 8) ? ((StringTag)tag).get() : defaultValue;
	}
	
	public default byte[] getByteArray(String id)
	{
		DataTag tag = get(id);
		return isType(tag, 9) ? ((ByteArrayTag)tag).get() : new byte[0];
	}
	
	public default short[] getShortArray(String id)
	{
		DataTag tag = get(id);
		return isType(tag, 10) ? ((ShortArrayTag)tag).get() : new short[0];
	}
	
	public default int[] getMediumArray(String id)
	{
		DataTag tag = get(id);
		return isType(tag, 11) ? ((MediumArrayTag)tag).get() : new int[0];
	}
	
	public default int[] getIntArray(String id)
	{
		DataTag tag = get(id);
		return isType(tag, 12) ? ((IntArrayTag)tag).get() : new int[0];
	}
	
	public default long[] getLongArray(String id)
	{
		DataTag tag = get(id);
		return isType(tag, 13) ? ((LongArrayTag)tag).get() : new long[0];
	}
	
	public default float[] getFloatArray(String id)
	{
		DataTag tag = get(id);
		return isType(tag, 14) ? ((FloatArrayTag)tag).get() : new float[0];
	}
	
	public default double[] getDoubleArray(String id)
	{
		DataTag tag = get(id);
		return isType(tag, 15) ? ((DoubleArrayTag)tag).get() : new double[0];
	}
	
	public default String[] getStringArray(String id)
	{
		DataTag tag = get(id);
		return isType(tag, 16) ? ((StringArrayTag)tag).get() : new String[0];
	}
	
	public default UUID getUUID(String id)
	{
		long[] data = getLongArray(id);
		return data == null || data.length != 2 ? null : new UUID(data[0], data[1]);
	}
	
	public default MapTag getMap(String id)
	{
		DataTag tag = get(id);
		return isType(tag, 17) ? (MapTag)tag : new MapTag();
	}
	
	public default ListTag getList(String id)
	{
		DataTag tag = get(id);
		return isType(tag, 18) ? (ListTag)tag : new ListTag();
	}
	
	public default boolean isPrimitve(DataTag tag)
	{
		int id = tag == null ? 0 : tag.getId();
		return id >= 1 && id <= 7;
	}
	
	public default boolean isType(DataTag tag, int type)
	{
		return (tag == null ? 0 : tag.getId()) == type;
	}
}